local ResultControls = {}
local ResultsSlider = nil
local GuildStoreSearchState = "NONE"
local StoreItems = {}
local SearchMatches = {}
local GuildStoreSearchTerms = nil
local MAX_GUILD_ID = 5
local SEARCH_DELAY = 3
local QualityColors = { None="|cCEDBBC", Normal="|cFEFEFE", Fine="|c2DC50E", Superior="|c3A92FF", Epic="|cA02EF7", Legendary="|cEECA2A" }

GSSEventHandlers = {}

local function ClearResultLine(index)
    if index < 1 or index > #ResultControls then
        return 
    end
    
    local resultControl = ResultControls[index]
    resultControl:SetText("")
    resultControl:GetNamedChild("Price"):SetText("")
    resultControl:GetNamedChild("UnitPrice"):SetText("")
    resultControl:GetNamedChild("BuyButton"):SetText("")

   
end

    
local function SetState(newState)
    GuildStoreSearchState = newState
end

local function ClearResultLines()
    for i = 1,#ResultControls do
        ClearResultLine(i)
    end
end


local function ClearResults()
    StoreItems = {}
    SearchMatches = {}
    GuildStoreSearchFindMatchesButton:SetHidden(true)
    GuildStoreSearchMatchCounter:SetText("")
    ClearResultLines()
end


local function GuildStoreSearchReset()
    if GuildStoreSearchTerms ~= nil then GuildStoreSearchTerms:SetText("") end
    GuildStoreSearchCounter:SetText("")
    ResultsSlider:SetMinMax(1,1)
    ResultsSlider:SetValue(1)
    ResultsSlider:SetHidden(true)
    ClearResults()
    SetState("NONE")
end


local function GetQualityColor(quality)

    if quality == ITEM_QUALITY_NORMAL then return QualityColors.Normal
    elseif quality == ITEM_QUALITY_MAGIC then return QualityColors.Fine
    elseif quality == ITEM_QUALITY_ARCANE then return QualityColors.Superior 
    elseif quality == ITEM_QUALITY_ARTIFACT then return QualityColors.Epic
    elseif quality == ITEM_QUALITY_LEGENDARY then return QualityColors.Legendary
    else return ""
    end
    
end


local function SetResultLine(index, name, quality, count, price, unitPrice, guildId, pageNumber)
    if index < 1 or index > #ResultControls then
        return 
    end

    local resultControl = ResultControls[index]
    
    local c = GetQualityColor(quality)
    local n = QualityColors.None 
    
    resultControl:SetText(string.format("G%s: %s[%s]%sx%s", guildId, c, name, n, count))
    resultControl:GetNamedChild("Price"):SetText(price.."g")
    resultControl:GetNamedChild("UnitPrice"):SetText(string.format("%.2f", unitPrice))
    resultControl:GetNamedChild("BuyButton"):SetText("Go to page")
    
end


local function DisplayMatches()

    local startIndex = ResultsSlider:GetValue()
    
    -- Don't scroll down lower than the last result
    if startIndex + #ResultControls - 1 > #SearchMatches then 
        startIndex = #SearchMatches - #ResultControls + 1 
    end
    
    -- Don't scroll up higher than the first result
    if startIndex < 1 then startIndex = 1 end
    
    for i = 1, #ResultControls do
        local resultIndex = i+startIndex-1
        if resultIndex > #SearchMatches then 
            ClearResultLine(i)
        else
            local r = SearchMatches[resultIndex]
            SetResultLine(i, r[2], r[3], r[4], r[7], r[8], r[9], r[10])
        end
    end

end


local function OnSliderMouseWheel(self, delta)

    if #SearchMatches <= #ResultControls then return end
    
    local oldSliderLevel = ResultsSlider:GetValue()
    local newSliderLevel = oldSliderLevel - delta

    -- local scrollUp = delta > 0
    -- local scrollDown = delta < 0
    
    ResultsSlider:SetValue(newSliderLevel)
end


local function OnSliderMoved(self,sliderLevel,eventReason)
    
    DisplayMatches()


end

local function CreateSlider(name, parent)

    local slider = CreateControl(name, parent, CT_SLIDER)
        
    local tex =  "/esoui/art/miscellaneous/scrollbox_elevator.dds"
		
    slider:SetDimensions(30,300)
	slider:SetMouseEnabled(true)
	slider:SetThumbTexture(tex,tex,tex,30,50,0,0,1,1)
    slider:SetMinMax(0,100)
	slider:SetValue(0)
	slider:SetValueStep(1)
	slider:SetAnchor(TOPRIGHT,parent,TOPRIGHT,0,0)
	
    return slider

end


local function GuildStoreSearchCommand(options)

	local parameters = GSSUtil.SplitString(options)

	if parameters[1] == "off" and GuildStoreSearch:IsHidden() == false then
		GuildStoreSearch:ToggleHidden()
        
	elseif parameters[1] ~= nil then
    
        if GuildStoreSearch:IsHidden() == true then
            -- Show the window 
            GuildStoreSearch:ToggleHidden()
        end
        
        GuildStoreSearchTerms:SetText(options)

    elseif GuildStoreSearch:IsHidden() == true and ZO_TradingHouse:IsHidden() == false then
    
        -- No parameters, just show the window
        --
        GuildStoreSearch:ToggleHidden()
    end
    
end



local function GuildStoreSearchToggle()
    if GuildStoreSearch:IsHidden() then
        GuildStoreSearchCommand()
    else
        GuildStoreSearchCommand("off")
    end
end


local function GuildStoreSearchInitialize(eventCode, addOnName)
    if(addOnName == "GuildStoreSearch") then
        
        if #ResultControls == 0 then
        
            local numberOfResultLines = 10
            local resultLineOffsetX = 0
            local resultLineOffsetY = 0
        
			for i = 1,numberOfResultLines do
				local control = CreateControlFromVirtual(
					"GuildStoreSearchResult", GuildStoreSearchResultsBG, "GuildStoreSearchResult", i)
				control:SetSimpleAnchorParent(resultLineOffsetX, resultLineOffsetY+((control:GetHeight()+2)*(i-1)))
				control:SetText(i .. ":")
                
				ResultControls[i] = control
			end
			
			local bg = CreateControlFromVirtual("GuildStoreSearchEditBg", GuildStoreSearch, "ZO_EditBackdrop")
			bg:SetParent(GuildStoreSearch)
			bg:SetDimensions(300,28)
			bg:SetSimpleAnchorParent(GuildStoreSearch:GetNamedChild("ForLabel"):GetWidth(), 5)

			GuildStoreSearchTerms = CreateControlFromVirtual("GuildStoreSearchTerms", bg, "ZO_DefaultEditForBackdrop")
			GuildStoreSearchTerms:SetParent(bg)
			GuildStoreSearchTerms:SetDimensions(300,28)
			GuildStoreSearchTerms:SetResizeToFitDescendents(false)
			GuildStoreSearchTerms:SetMouseEnabled(true)
			GuildStoreSearchTerms:SetText("")       
			
			local gdd = CreateControlFromVirtual("GuildStoreSearchGuildCombo", GuildStoreSearch, "GuildStoreSearchDropdown")
			gdd:SetDimensions(300,28)
			gdd:SetSimpleAnchorParent(GuildStoreSearch:GetNamedChild("GuildLabel"):GetWidth(), bg:GetHeight() + 10 )

            GuildStoreSearchResultsBG:SetMouseEnabled(true)
            GuildStoreSearchResultsBG:SetHandler("OnMouseWheel", OnSliderMouseWheel)
            
            ResultsSlider = CreateSlider("GuildStoreSearchSlider", GuildStoreSearch)
            ResultsSlider:SetAnchor(LEFT,GuildStoreSearchResultsBG,RIGHT,0,0)
            ResultsSlider:SetHandler("OnValueChanged", OnSliderMoved)
            
            local pane = ZO_TradingHouseLeftPaneBrowseItemsCommon

            local toggleButton = CreateControl("GuildStoreSearchToggleButton", pane, CT_BUTTON)
            toggleButton:SetDimensions(195,31)
            toggleButton:SetHidden(false)
            toggleButton:SetFont("ZoFontGameBold")
            toggleButton:SetVerticalAlignment(TEXT_ALIGN_CENTER)
            toggleButton:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
            toggleButton:SetText("Guild Store Search")
            --toggleButton:SetNormalTexture("esoui/art/tradinghouse/tradinghouse_browse_tabicon_up.dds")
            toggleButton:SetAnchor(TOPLEFT, pane:GetNamedChild("Quality"), TOPLEFT, 0, 120)
            toggleButton:SetHandler("OnMouseDown", GuildStoreSearchToggle)
            
            bg = CreateControlFromVirtual("GuildStoreSearchToggleButtonBG", toggleButton, "GuildStoreSearchVirtualBG")
            bg:SetAnchorFill()
            
		end

        GuildStoreSearchReset()

    end
end


local function CollateResults(guildId, numItemsOnPage, currentPage, hasMorePages)

    for i = 1, numItemsOnPage do
    
        local icon, name, quality, stackCount, seller, timeRemaining, price = 
            GetTradingHouseSearchResultItemInfo(i)

        local currentResult = {icon, name, quality, tonumber(stackCount), seller, 
                                tonumber(timeRemaining), tonumber(price), 
                                tonumber(price)/tonumber(stackCount),
                                guildId, currentPage}

        table.insert(StoreItems, currentResult)

    end
end


local function StrictCompareByUnitPrice(a,b)
  return a[8] < b[8]
end

local function WaitThenSearch(delayInSec, page, sortBy)
  
  zo_callLater(function() ExecuteTradingHouseSearch(page,sortBy, true) end, delayInSec * 1000)

end

local function GuildStoreSearchResultsReceived (eventId, guildId, numItemsOnPage, currentPage, hasMorePages)
    if GuildStoreSearchState == "AWAITING_RESULTS" then 

        CollateResults(guildId, numItemsOnPage, currentPage, hasMorePages)
    
        if (hasMorePages) then
            WaitThenSearch(SEARCH_DELAY, currentPage+1,TRADING_HOUSE_SORT_SALE_PRICE)
            GuildStoreSearchCounter:SetText(string.format("Searching Guild %s, page %s", guildId, currentPage+1))

        else
            local resultCount = table.getn(StoreItems)
            GuildStoreSearchCounter:SetText(string.format("Found %s items from Guild %s", resultCount, guildId))
            GuildStoreSearchFindMatchesButton:SetHidden(false)
            SetState("NONE")
        end
    
    elseif GuildStoreSearchState == "AWAITING_RESULTS_ALL" then
    
        CollateResults(guildId, numItemsOnPage, currentPage, hasMorePages)
    
        if (hasMorePages) then
            WaitThenSearch(SEARCH_DELAY, currentPage+1,TRADING_HOUSE_SORT_SALE_PRICE)
            GuildStoreSearchCounter:SetText(string.format("Retrieving from Guild %s, page %s", guildId, currentPage+1))

        elseif guildId < MAX_GUILD_ID then
            local canBuyFrom = true
            
            while guildId < MAX_GUILD_ID do
                canBuyFrom = CanBuyFromTradingHouse(guildId + 1)

                if (canBuyFrom) then
                    SelectTradingHouseGuildId(guildId + 1)
                    WaitThenSearch(SEARCH_DELAY,0,TRADING_HOUSE_SORT_SALE_PRICE)
                    break
                else
                    guildId = guildId + 1
                end
            end
            
            if canBuyFrom == false then
                local resultCount = table.getn(StoreItems)
                GuildStoreSearchCounter:SetText(string.format("Found %s items from all guilds ", resultCount))
                GuildStoreSearchFindMatchesButton:SetHidden(false)
                SetState("NONE")
            end

        else 
            local resultCount = table.getn(StoreItems)
            GuildStoreSearchCounter:SetText(string.format("Found %s items from all guilds", resultCount))
            GuildStoreSearchFindMatchesButton:SetHidden(false)
            SetState("NONE")
            
        end
    end
    
end


local function GuildStoreClosed()
    if  GuildStoreSearch:IsHidden() == false then
        GuildStoreSearch:ToggleHidden()
    end
end


local function PopulateGuildList()
    local dropdown = ZO_ComboBox_ObjectFromContainer(GuildStoreSearch:GetNamedChild("GuildCombo"))
    local guilds = {}
    
    dropdown:ClearItems()
    
    local entry = dropdown:CreateItemEntry("All", function() end)
    dropdown:AddItem(entry)

    for i = 1,MAX_GUILD_ID do
        local id,name,alliance = GetTradingHouseGuildDetails(i)
        if name == nil or name == "" then break end
        
        local entry = dropdown:CreateItemEntry(string.format("G%s: %s", id, name), function() end)
        dropdown:AddItem(entry)
        
        guilds[id] = name

    end
    
    local selected = GetSelectedTradingHouseGuildId()
    dropdown:SetSelectedItem(string.format("G%s: %s", selected, guilds[selected])) 
end


local function GuildStoreStatusReceived()
    PopulateGuildList()
end


local function GetGuildIdFromDropdown()
    local dropdown = ZO_ComboBox_ObjectFromContainer(GuildStoreSearch:GetNamedChild("GuildCombo"))
    local guildId = dropdown:GetSelectedItem()

    if guildId ~= "All" then 
        guildId = string.sub(guildId, 2, 2)
    end
    
    return guildId
end

local function GuildStoreSearchRetrieve()    
    if GuildStoreSearchState ~= "NONE" then return end
    
    GuildStoreSearchFindMatchesButton:SetHidden(true)

    ClearResults()

    selectedText = GetGuildIdFromDropdown()

    if selectedText == "All" then 
        SelectTradingHouseGuildId(1)
    else
        SelectTradingHouseGuildId(tonumber(selectedText))
    end

    ExecuteTradingHouseSearch(0,TRADING_HOUSE_SORT_SALE_PRICE, true)
    
    if selectedText == "All" then 
        SetState("AWAITING_RESULTS_ALL")
    else
        SetState("AWAITING_RESULTS")
    end
    
end


local function GuildStoreSearchFindMatches()
    if StoreItems == nil or table.getn(StoreItems) < 1 then return end
    
    local searchTerms = GSSUtil.Trim(string.lower(GuildStoreSearchTerms:GetText()))
    if searchTerms == "" then return end

    ClearResultLines()

    local matches = {}
    local exact = searchTerms:sub(1,1) == "\"" and searchTerms:sub(-1,-1) == "\""
    
    if searchTerms == "*" then 
        matches = StoreItems
    
    else

        for i = 1, #StoreItems do
        
            local currentResult = StoreItems[i]
            -- icon, name, quality, stackCount, seller, timeRemaining, 
            -- price, unitPrice, guildId, resultPage 
            
            local currentGuild = GetGuildIdFromDropdown()
        
            local name = currentResult[2]
            local guildId = tostring(currentResult[9])
            
            if currentGuild == "All" then currentGuild = guildId end

            if exact then
            
                -- Some item names have ^n or ^p at the end.
                -- Remove them if doing an exact search
                if name:sub(-2,-2) == "^" then
                    name = name:sub(1,-3)
                end
                
                -- Remove quotes from search terms
                local unquotedSearchTerms = searchTerms:sub(2,-2)
                
                if string.lower(name) == unquotedSearchTerms and currentGuild == guildId then
                    table.insert(matches, currentResult)
                end

            else
            
                findResult = string.lower(name):find(searchTerms)

                if findResult ~= nil and currentGuild == guildId then
                    table.insert(matches, currentResult)
                end
            end
        end
    end

    table.sort(matches, StrictCompareByUnitPrice)
    
    matchCount = table.getn(matches)
    
    SearchMatches = matches
    
    GuildStoreSearchMatchCounter:SetText(string.format("%s matches", matchCount))
    if #SearchMatches <= #ResultControls then
        ResultsSlider:SetMinMax(1, 1)
        ResultsSlider:SetHidden(true)
    else
        ResultsSlider:SetMinMax(1, #SearchMatches - #ResultControls + 1)
        ResultsSlider:SetHidden(false)
    end
    ResultsSlider:SetValue(1)
    
    DisplayMatches()
end

local function ShowItem(itemButton)
    local start = 23 -- length of "GuildStoreSearchResult" + 1
    local scrollOffset = ResultsSlider:GetValue() - 1
    local displayRow = tonumber(string.sub(itemButton:GetParent():GetName(), 23, -1)) 
    local resultIndex = displayRow + scrollOffset
    
    local result = SearchMatches[ resultIndex ]
    
    if result == nil then return end
    
    local guildId = result[9]
    local resultPage = result[10]
        
    SelectTradingHouseGuildId(guildId)
    ExecuteTradingHouseSearch(resultPage,TRADING_HOUSE_SORT_SALE_PRICE, true)

end    


SLASH_COMMANDS["/gss"] = GuildStoreSearchCommand

EVENT_MANAGER:RegisterForEvent("GuildStoreSearch", EVENT_ADD_ON_LOADED, GuildStoreSearchInitialize)
EVENT_MANAGER:RegisterForEvent("GuildStoreSearch", EVENT_TRADING_HOUSE_SEARCH_RESULTS_RECEIVED, GuildStoreSearchResultsReceived)
EVENT_MANAGER:RegisterForEvent("GuildStoreSearch", EVENT_TRADING_HOUSE_STATUS_RECEIVED, GuildStoreStatusReceived)
EVENT_MANAGER:RegisterForEvent("GuildStoreSearch", EVENT_CLOSE_TRADING_HOUSE, GuildStoreClosed)


-- Provide public exposure to methods so they 
-- can be linked directly in XML UI markup
GSSEventHandlers.Reset = GuildStoreSearchReset
GSSEventHandlers.ShowItem = ShowItem
GSSEventHandlers.Close = GuildStoreClosed
GSSEventHandlers.Retrieve = GuildStoreSearchRetrieve
GSSEventHandlers.FindMatches = GuildStoreSearchFindMatches

